#!/usr/bin/python
# -*- coding: UTF-8 -*-
'''
This is a simple code to help with Dovecot Public Shared Mailbox function.
Just configure a public shared mailbox with ACLs, change the configuration file
and run this program.

How it works:
1) The program connect to the LDAP server and fetch groups from it (with ldap filter)
2) With groups, it fetches the members and owners (or any attribute you want);
3) It will walk into the "shared folder" folder base directory and:
  3.1) Rename any directory that was renamed in LDAP (based on entryUUID ldap attribute);
  3.2) Create diretories and skeletion (new, cur and tmp) of any group that exists in LDAP;
  3.3) Optionaly, remove any "orphan" directory (wihtout a ldap entrey);

Example:
dn: cn=Everyone,ou=Shared_Mailbox,dc=example,dc=com
objectClass: labeledURIObject
objectClass: top
objectClass: groupOfURLs
cn: Everyone
labeledURI: ldap:///ou=Users,dc=example,dc=com?uid?one?(objectClass=person)
member: uid=user2,ou=users,dc=example,dc=com
member: uid=user3,ou=users,dc=example,dc=com
member: uid=user1,ou=users,dc=example,dc=com
member: uid=user.surname,ou=users,dc=example,dc=com
owner: uid=user4,ou=Users,dc=example,dc=com

This will create a folder tree like this:
  /path/toshared/.Everyone
  /path/toshared/.Everyone/new
  /path/toshared/.Everyone/cur
  /path/toshared/.Everyone/tmp
With permission file (/path/toshared/.Everyone/dovecot-acl):
  user=user1 lr
  ...
  user=user.surname lr
  user=user4 lrwstipekxa

Hope this helps you! Enjoy.

Dovecot Public Shared MailBox:
http://wiki2.dovecot.org/SharedMailboxes/Public

Dovecot: http://dovecot.org
GPL: http://www.gnu.org/copyleft/gpl.html
'''
__author__	= 'Allan GooD mail: allan.cassaro_at_gmail.com'
__copyright__	= 'GPLv3'

import os
import re
import sys
import ldap
import ConfigParser

'''
Simple function to delete a complete directory.
Yes, I know, its possible to do with shutil, but I want to use an already loaded module (os)
'''

def print_help():
  print('You need to inform a configuration file.')
  print('%s /path/to/config.file' % sys.argv[0])

def rmtree(path):
  for root, dirs, files in os.walk(path, topdown=False):
    for name in files:
      os.remove(os.path.join(root, name))
    for name in dirs:
      os.rmdir(os.path.join(root, name))
  os.rmdir(path)

if (len(sys.argv) != 2):
  print_help()
  sys.exit(-1)

configfile = sys.argv[1]
config = ConfigParser.ConfigParser()
config.read(configfile)

try:
  ldap_server = config.get('ldap','server')
  login_dn = config.get('ldap','binddn')
  login_pw = config.get('ldap','bindpw')
  basedn = config.get('ldap','basedn')
  group_filter = config.get('ldap','group_filter')
  group_memberattr = config.get('ldap','group_memberattr')
  group_adminattr = config.get('ldap','group_adminattr')
  group_nameattr = config.get('ldap','group_nameattr')
  retrieve_attributes = [group_memberattr,group_adminattr,group_nameattr, 'entryUUID']
  home_base = config.get('system','home_base')
  remove_orphan = config.get('system','remove_orphan_dir')
except ConfigParser.NoOptionError as e:
  print('Arquivo de configuracoes com erros!\nErro: %s' %(e))
  sys.exit(-1)

'''
First, look for changed names and rename directories
'''
for name in os.listdir(home_base):
  uuidfilename = home_base + os.sep + name + os.sep + '.ldap_uuid'
  os.path.exists(uuidfilename)
  uuidfile = open(uuidfilename,'r')
  uuid = uuidfile.readline().strip('\n')
  uuidfile.close()
  cn = None
  try:
    ldapfilter = 'entryUUID=%s' % uuid
    l = ldap.open(ldap_server)
    l.simple_bind_s(login_dn, login_pw)
    result_id = l.search(basedn, ldap.SCOPE_SUBTREE, ldapfilter, ['cn'])
    result_type, ldap_response = l.result(result_id, 5)
    if (len(ldap_response) > 0):
      cn = ldap_response[0][1]['cn'][0]
  except ldap.LDAPError, error_message:
    print('Impossivel conectar. %s ' % error_message)
    sys.exit(-1)
  if (cn is not None) and (name[1:] != cn):
    os.rename(home_base + os.sep + name,home_base + os.sep + '.' + cn)


'''
Now create new
'''
try:
  l = ldap.open(ldap_server)
  l.simple_bind_s(login_dn, login_pw)
  result_id = l.search(basedn, ldap.SCOPE_SUBTREE, group_filter, retrieve_attributes)
  result_type, ldap_response = l.result(result_id, 5)
except ldap.LDAPError, error_message:
  print('Impossivel conectar. %s ' % error_message)
  sys.exit(-1)

dict_groups = {}
for i in ldap_response:
  group_name = None
  group_uuid = None
  group_members = []
  group_owners = []
  if (group_nameattr in i[1].keys()):
    group_name = i[1][group_nameattr][0]
  if (group_memberattr in i[1].keys()):
    group_members = i[1][group_memberattr]
  if (group_adminattr in i[1].keys()):
    group_owners = i[1][group_adminattr]
  if ('entryUUID' in i[1].keys()):
    group_uuid = i[1]['entryUUID'][0]
  if (group_name is not None):
    dict_groups[group_name] = {'members': group_members, 'owners': group_owners, 'uuid': group_uuid} 

skel = ['tmp','cur','new']
for folder in dict_groups.keys():
  dir_name = '%s%s.%s' % (home_base,os.sep,folder)
  if not os.path.exists(dir_name):
    os.makedirs(dir_name)
    for subdir in skel:
      os.makedirs(dir_name + os.sep + subdir)
  else:
    for subdir in skel:
      if not os.path.exists(dir_name + os.sep + subdir):
        os.makedirs(dir_name + os.sep + subdir)
  with open(dir_name + os.sep + '.ldap_uuid','w') as uuidfile:
    uuidfile.write('%s\n' % dict_groups[folder]['uuid'])
  with open(dir_name + os.sep + 'dovecot-acl','w') as aclfile:
    for user in dict_groups[folder]['members']:
      if user not in dict_groups[folder]['owners']:
        p = re.compile('(?:cn|ou|dc|uid=([^,"]+))',re.IGNORECASE)
        user = p.match(user).group(1).lower()
        aclfile.write('user=%s lr\n' % user)
    for user in dict_groups[folder]['owners']:
      p = re.compile('(?:cn|ou|dc|uid=([^,"]+))',re.IGNORECASE)
      user = p.match(user).group(1).lower()
      aclfile.write('user=%s lrwstipekxa\n' % user)


'''
Finaly, remove (with you want) the orphan ones
'''
if (remove_orphan.lower() == 'yes'):
  for name in os.listdir(home_base):
    if os.path.isdir(home_base + os.sep + name):
      if name[1:] not in dict_groups.keys():
        rmtree(home_base + os.sep + name)
